<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\Esb\Model\Rest;
use FiloBlu\Esb\Api\ExternalShipmentInterface;
use Magento\Sales\Api\Data\ShipmentInterface;

/**
 * REST Api to save external shipments
 */
class ExternalShipment implements ExternalShipmentInterface{
	
    /**
     * @var \Magento\Sales\Model\OrderRepository
     */ 	
    protected $_orderRepository;

    /**
     * @var \Magento\Sales\Model\Order\ShipmentFactory 
     */ 	
    protected $_shipmentFactory;
    
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder 
     */ 
    protected $_searchCriteriaBuilder;

    /**
     * @var \Magento\Sales\Model\Order\ShipmentFactory 
     */ 	
    protected $_shipmentNotifier;
    
    
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $_orderExtensionFactory;


    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Sales\Model\Order\ShipmentFactory $shipmentFactory     
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder     
     */
    public function __construct(
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \Magento\Sales\Model\OrderRepository $orderRepository,
            \Magento\Sales\Model\Order\ShipmentFactory $shipmentFactory,
            \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
            \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier,
            \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
    ){
        $this->_scopeConfig = $scopeConfig;
        $this->_orderRepository = $orderRepository;
        $this->_shipmentFactory = $shipmentFactory;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_shipmentNotifier = $shipmentNotifier;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;              
    }

    /**
     * Save External Shipment data. Save PDF file and additional data on Sales Order.
     *
     * @param \Magento\Sales\Api\Data\ShipmentInterface  $shipment_data
     * @return int
     */
     public function save(ShipmentInterface $shipment_data){
        $shipmentExtensionAttributes=$shipment_data->getExtensionAttributes();
        if(null==$shipmentExtensionAttributes || !$shipmentExtensionAttributes->getOrderIncrementId()){
            return 0;
        }
        $orderIncrementId=$shipmentExtensionAttributes->getOrderIncrementId();
        $carrierCode=$shipmentExtensionAttributes->getCarrierCode();
        if(!$carrierCode) $carrierCode="ups";
        $carrierTitle=$shipmentExtensionAttributes->getCarrierTitle();
        if(!$carrierTitle) $carrierTitle="UPS";
        $searchCriteria = $this->_searchCriteriaBuilder
            ->addFilter('increment_id', $orderIncrementId, 'eq')->create();
        $orderList = $this->_orderRepository->getList($searchCriteria);
        if(count($orderList)<>1){
            return 0;
        }
        try{
            $order=$orderList->getFirstItem();
            $track=current($shipment_data->getTracks());
            $shipment=$this->_shipmentFactory->create(
                $order,
                array(),
                array(array(
                    "order_id" => $order->getId(),
                    "carrier_code" => $carrierCode,
                    "title" => $carrierTitle,                    
                    "number" => $track->getTrackNumber()
                    )
                )
            );
            $shipment->register();
            $shipment->getOrder()->setIsInProcess(true);
            $shipment->save();
            $order->addStatusHistoryComment(__("Order has been shipped: %1 ", $track->getTrackNumber()), "shipped");
            $order->save();
            $notify=$this->_shipmentNotifier->notify($shipment);
            $shipment->save();
            return $shipment->getId();
        }catch(Exception $e){
            return 0;
        }
    }
}

